# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

from singleton_decorator import singleton

from aisecurity.exceptions import AISecSDKException, ErrorType
from aisecurity.generated_openapi_client.models.async_scan_object import AsyncScanObject
from aisecurity.generated_openapi_client.models.async_scan_response import (
    AsyncScanResponse,
)
from aisecurity.generated_openapi_client.urllib3.exceptions import ApiException
from aisecurity.scan.inline.base import ScanApiBase


@singleton
class AsyncScanExecutor(ScanApiBase):
    def __init__(self):
        """
        Initialize the AsyncScanExecutor with the ScansApi client.
        """
        super().__init__()

    def async_request(self, scan_objects: list[AsyncScanObject]) -> AsyncScanResponse:
        """
        Make an asynchronous scan request to the API.

        Args:
            scan_objects (list[AsyncScanObject]): A list of AsyncScanObjects to be scanned.

        Returns:
            AsyncScanResponse: The response from the asynchronous scan request.

        Raises:
            Exception: If there's an error during the API call.
        """
        if scan_objects is None or len(scan_objects) == 0:
            error_msg = "No scan objects are provided."
            self.logger.error(f"event={self.async_request.__name__} error={error_msg}")
            raise AISecSDKException(error_msg, ErrorType.USER_REQUEST_PAYLOAD_ERROR)

        try:
            response = self.scan_api.scan_async_request(async_scan_object=scan_objects)
            return response
        except ApiException as e:
            """
                Handle API exceptions that may occur during the API call. Ex HTTP Server side Errors,API Key errors,API value errors,Authorization errors etc.
            """
            self.logger.error(f"event={self.async_request.__name__} error={e}")
            raise AISecSDKException(str(e), ErrorType.SERVER_SIDE_ERROR)
        except Exception as e:
            """
                A catch-all for any unexpected exceptions on the Wrapper side.
            """
            self.logger.error(f"event={self.async_request.__name__} error={e}")
            raise AISecSDKException(str(e), ErrorType.AISEC_SDK_ERROR)



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

"""
AISec API service

OpenAPI Specification for the AI Runtime Security API service

The version of the OpenAPI document: 0.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from aisecurity.generated_openapi_client import ScanIdResult
from aisecurity.generated_openapi_client.asyncio.api_client import (
    ApiClient,
    RequestSerialized,
)
from aisecurity.generated_openapi_client.asyncio.api_response import ApiResponse
from aisecurity.generated_openapi_client.asyncio.rest import RESTResponseType


class ScanResultsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    async def get_scan_results_by_scan_ids(
        self,
        scan_ids: Annotated[List[StrictStr], Field(description="Scan Ids for Results")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ScanIdResult]:
        """Retrieve Scan Results by ScanIDs

        Get the Scan results for upto a maximum of 5 Scan IDs

        :param scan_ids: Scan Ids for Results (required)
        :type scan_ids: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_scan_results_by_scan_ids_serialize(
            scan_ids=scan_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ScanIdResult]",
        }
        response_data = await self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_scan_results_by_scan_ids_with_http_info(
        self,
        scan_ids: Annotated[List[StrictStr], Field(description="Scan Ids for Results")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ScanIdResult]]:
        """Retrieve Scan Results by ScanIDs

        Get the Scan results for upto a maximum of 5 Scan IDs

        :param scan_ids: Scan Ids for Results (required)
        :type scan_ids: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_scan_results_by_scan_ids_serialize(
            scan_ids=scan_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ScanIdResult]",
        }
        response_data = await self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_scan_results_by_scan_ids_without_preload_content(
        self,
        scan_ids: Annotated[List[StrictStr], Field(description="Scan Ids for Results")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve Scan Results by ScanIDs

        Get the Scan results for upto a maximum of 5 Scan IDs

        :param scan_ids: Scan Ids for Results (required)
        :type scan_ids: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_scan_results_by_scan_ids_serialize(
            scan_ids=scan_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ScanIdResult]",
        }
        response_data = await self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _get_scan_results_by_scan_ids_serialize(
        self,
        scan_ids,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "scan_ids": "csv",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if scan_ids is not None:
            _query_params.append(("scan_ids", scan_ids))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/v1/scan/results",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.
import asyncio
from typing import Optional

from aisecurity.generated_openapi_client import (
    AiProfile,
    AsyncScanObject,
    AsyncScanResponse,
    ScanIdResult,
    ScanResponse,
    ThreatScanReportObject,
)
from aisecurity.generated_openapi_client.models.metadata import Metadata
from aisecurity.logger import BaseLogger
from aisecurity.scan.asyncio.async_scan_executor import AsyncScanExecutor
from aisecurity.scan.asyncio.query_by_report_ids import QueryByReportIds
from aisecurity.scan.asyncio.query_by_scan_ids import QueryByScanIds
from aisecurity.scan.asyncio.scan_executor import ScanExecutor
from aisecurity.scan.models.content import Content


class Scanner(BaseLogger):
    """
    A class to handle various scanning operations and queries.
    """

    def __init__(self):
        """
        Initialize the Scanner object with default values and logger setup.
        """
        super().__init__()
        self._scan_executor = None
        self._async_scan_executor = None
        self._query_scan_by_scan_ids = None
        self._query_by_report_ids = None

    async def sync_scan(
        self,
        ai_profile: AiProfile,
        content: Content,
        tr_id: Optional[str] = None,
        session_id: Optional[str] = None,
        metadata: Optional[Metadata] = None,
    ) -> ScanResponse:
        """
        Perform a synchronous (inline) scan.

        Args:
            ai_profile (AiProfile): The AI profile to use for the scan.
            content (Content): The content to be scanned.
            tr_id (str): Optionally Provide any unique identifier string for correlating the prompt and \
                        response transactions. This is an optional field. The tr_id value received for scan request is \
                        returned in the scan response along with the scan ID
            session_id (str): Optionally send session_id to track session views
            metadata (Metadata): Optionally send the app_name, app_user, and ai_model in the metadata

        Returns:
            scan response and of the operation.
        """
        if self._scan_executor is None:
            self._scan_executor = ScanExecutor()

        scan_response = await self._scan_executor.sync_request(
            ai_profile=ai_profile, content=content, tr_id=tr_id, session_id=session_id, metadata=metadata
        )

        self.logger.info(f"event={self.sync_scan.__name__} ")
        self.logger.debug(f"event={self.sync_scan.__name__} ai_profile={ai_profile!r} ")

        return scan_response

    async def async_scan(self, async_scan_objects: list[AsyncScanObject]) -> AsyncScanResponse:
        """
        Perform an asynchronous scan.

        Args:
            async_scan_objects (list[AsyncScanObject]): A list of AsyncScanObject instances to be scanned.

        Returns:
            Response containing the AsyncScanResponse object
        """
        if self._async_scan_executor is None:
            self._async_scan_executor = AsyncScanExecutor()

        async_scan_response = await self._async_scan_executor.async_request(
            scan_objects=async_scan_objects,
        )

        self.logger.info(f"event={self.async_scan.__name__} ")

        return async_scan_response

    async def query_by_scan_ids(self, scan_ids: list[str]) -> list[ScanIdResult]:
        """
        Query scan results by scan IDs.

        Args:
            scan_ids (list[str]): A list of scan IDs to query.

        Returns:
            List of scan ID's responses.
        """
        if self._query_scan_by_scan_ids is None:
            self._query_scan_by_scan_ids = QueryByScanIds()

        scan_id_responses = await self._query_scan_by_scan_ids.get_scan_results(scan_ids=scan_ids)

        self.logger.info(f"event={self.query_by_report_ids.__name__} ")
        self.logger.debug(f"event={self.query_by_report_ids.__name__} scan_ids={scan_ids!r} ")

        return scan_id_responses

    async def query_by_report_ids(self, report_ids: list[str]) -> list[ThreatScanReportObject]:
        """
        Query scan results by report IDs.

        Args:
            report_ids (list[str]): A list of report IDs to query.

        Returns:
           List of ThreatReportObjects.
        """
        if self._query_by_report_ids is None:
            self._query_by_report_ids = QueryByReportIds()

        scan_id_responses = await self._query_by_report_ids.get_threat_objects(report_ids=report_ids)

        self.logger.info(f"event={self.query_by_report_ids.__name__} ")
        self.logger.debug(f"event={self.query_by_report_ids.__name__} report_ids={report_ids!r} ")

        return scan_id_responses

    async def close(self) -> None:
        """
        Closes all Open Asyncio Connection Pools in underlying HTTP client libraries.

        Use this function to avoid leaking asyncio tasks, which may result in a warning emitted to stderr.
        """
        executors = [
            self._scan_executor,
            self._async_scan_executor,
            self._query_scan_by_scan_ids,
            self._query_by_report_ids,
        ]
        tasks = []
        for executor in executors:
            if executor is not None:
                self.logger.debug(f"Closing connection pool for {executor.__class__.__name__}")
                tasks.append(executor.close())
        if tasks:
            await asyncio.gather(*tasks)
            self.logger.debug(f"Closed {len(tasks)} asyncio connection pools")



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

# flake8: noqa

"""
AISec API service

OpenAPI Specification for the AI Runtime Security API service

The version of the OpenAPI document: 0.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

__version__ = "1.0.0"

# import apis into sdk package
from aisecurity.generated_openapi_client.urllib3.api import ScanReportsApi

# import ApiClient
from aisecurity.generated_openapi_client.urllib3.api_response import ApiResponse

# import models into sdk package
from aisecurity.generated_openapi_client.models.ai_profile import AiProfile
from aisecurity.generated_openapi_client.models.async_scan_object import AsyncScanObject
from aisecurity.generated_openapi_client.models.async_scan_response import (
    AsyncScanResponse,
)
from aisecurity.generated_openapi_client.models.ds_detail_result_object import (
    DSDetailResultObject,
)
from aisecurity.generated_openapi_client.models.detection_service_result_object import (
    DetectionServiceResultObject,
)
from aisecurity.generated_openapi_client.models.dlp_report_object import DlpReportObject
from aisecurity.generated_openapi_client.models.error import Error
from aisecurity.generated_openapi_client.models.metadata import Metadata
from aisecurity.generated_openapi_client.models.prompt_detected import PromptDetected
from aisecurity.generated_openapi_client.models.response_detected import (
    ResponseDetected,
)
from aisecurity.generated_openapi_client.models.scan_id_result import ScanIdResult
from aisecurity.generated_openapi_client.models.scan_request import ScanRequest
from aisecurity.generated_openapi_client.models.scan_request_contents_inner import (
    ScanRequestContentsInner,
)
from aisecurity.generated_openapi_client.models.scan_response import ScanResponse
from aisecurity.generated_openapi_client.models.scan_sync_request_default_response import (
    ScanSyncRequestDefaultResponse,
)
from aisecurity.generated_openapi_client.models.threat_scan_report_object import (
    ThreatScanReportObject,
)
from aisecurity.generated_openapi_client.models.urlf_entry_object import UrlfEntryObject



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

import hashlib
import hmac
import json
import uuid
from typing import Any, Union


class Utils:
    @staticmethod
    def is_valid_uuid(id_string: str) -> bool:
        """
        Check if a given string is a valid UUID.

        Args:
        id_string (str): The string to check.

        Returns:
        bool: True if the string is a valid UUID, False otherwise.
        """
        try:
            uuid_obj = uuid.UUID(id_string)
            return str(uuid_obj) == id_string.lower()
        except ValueError:
            return False

    @staticmethod
    def generate_payload_hash(api_key: str, body: Any) -> str:
        """
        Generate an HMAC-SHA256 hash of the request body using the API key as the secret.

        Args:
            api_key: The API key used as the secret key
            body: The request body to be hashed

        Returns:
            str: Hexadecimal digest of the HMAC hash
        """
        try:
            body_str = json.dumps(body)
            digest = hmac.new(
                key=api_key.encode("utf-8"),
                msg=body_str.encode("utf-8"),
                digestmod=hashlib.sha256,
            ).hexdigest()
            return digest
        except (TypeError, ValueError, ImportError) as e:
            raise e


def safe_flatten(maybe_list_of_lists: list[Union[list[Any], Any]]) -> list[Any]:
    """
    Safely flattens a list that may contain nested lists.

    Args:
        maybe_list_of_lists (list[Union[list[Any], Any]]): A list that may contain nested lists.

    Returns:
        list[Any]: A flattened list containing all elements.

    Raises:
        ValueError: If the input is not a list.
    """
    new_list: list[Any] = []
    if not isinstance(maybe_list_of_lists, list):
        raise ValueError(f"Expected a list, got {type(maybe_list_of_lists)}")
    for entry in maybe_list_of_lists:
        if isinstance(entry, list):
            new_list.extend(entry)
        else:
            new_list.append(entry)
    return new_list



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

"""
AISec API service

OpenAPI Specification for the AI Runtime Security API service

The version of the OpenAPI document: 0.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from aisecurity.generated_openapi_client.models.masked_data import MaskedData
from aisecurity.generated_openapi_client.models.prompt_detected import PromptDetected
from aisecurity.generated_openapi_client.models.prompt_detection_details import PromptDetectionDetails
from aisecurity.generated_openapi_client.models.response_detected import ResponseDetected
from aisecurity.generated_openapi_client.models.response_detection_details import ResponseDetectionDetails
from aisecurity.generated_openapi_client.models.tool_detected import ToolDetected
from typing import Optional, Set
from typing_extensions import Self


class ScanResponse(BaseModel):
    """
    ScanResponse
    """  # noqa: E501

    source: Optional[StrictStr] = Field(
        default=None, description="Source of the scan request (e.g., 'AI-Runtime-MCP-Server' or 'AI-Runtime-API')"
    )
    report_id: StrictStr = Field(description="Unique identifier for the scan report")
    scan_id: StrictStr = Field(description="Unique identifier for the scan")
    tr_id: Optional[StrictStr] = Field(default=None, description="Unique identifier for the transaction")
    session_id: Optional[StrictStr] = Field(default=None, description="Unique identifier for tracking Sessions")
    profile_id: Optional[StrictStr] = Field(
        default=None, description="Unique identifier of the AI security profile used for scanning"
    )
    profile_name: Optional[StrictStr] = Field(default=None, description="AI security profile name used for scanning")
    category: StrictStr = Field(description='Category of the scanned content verdicts such as "malicious" or "benign"')
    action: StrictStr = Field(
        description='The action is set to "block" or "allow" based on AI security profile used for scanning'
    )
    prompt_detected: Optional[PromptDetected] = None
    response_detected: Optional[ResponseDetected] = None
    prompt_masked_data: Optional[MaskedData] = None
    response_masked_data: Optional[MaskedData] = None
    prompt_detection_details: Optional[PromptDetectionDetails] = None
    response_detection_details: Optional[ResponseDetectionDetails] = None
    tool_detected: Optional[ToolDetected] = None
    created_at: Optional[datetime] = Field(default=None, description="Scan request timestamp")
    completed_at: Optional[datetime] = Field(default=None, description="Scan completion timestamp")
    __properties: ClassVar[List[str]] = [
        "source",
        "report_id",
        "scan_id",
        "tr_id",
        "session_id",
        "profile_id",
        "profile_name",
        "category",
        "action",
        "prompt_detected",
        "response_detected",
        "prompt_masked_data",
        "response_masked_data",
        "prompt_detection_details",
        "response_detection_details",
        "tool_detected",
        "created_at",
        "completed_at",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ScanResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of prompt_detected
        if self.prompt_detected:
            _dict["prompt_detected"] = self.prompt_detected.to_dict()
        # override the default output from pydantic by calling `to_dict()` of response_detected
        if self.response_detected:
            _dict["response_detected"] = self.response_detected.to_dict()
        # override the default output from pydantic by calling `to_dict()` of prompt_masked_data
        if self.prompt_masked_data:
            _dict["prompt_masked_data"] = self.prompt_masked_data.to_dict()
        # override the default output from pydantic by calling `to_dict()` of response_masked_data
        if self.response_masked_data:
            _dict["response_masked_data"] = self.response_masked_data.to_dict()
        # override the default output from pydantic by calling `to_dict()` of prompt_detection_details
        if self.prompt_detection_details:
            _dict["prompt_detection_details"] = self.prompt_detection_details.to_dict()
        # override the default output from pydantic by calling `to_dict()` of response_detection_details
        if self.response_detection_details:
            _dict["response_detection_details"] = self.response_detection_details.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tool_detected
        if self.tool_detected:
            _dict["tool_detected"] = self.tool_detected.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ScanResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "source": obj.get("source"),
            "report_id": obj.get("report_id"),
            "scan_id": obj.get("scan_id"),
            "tr_id": obj.get("tr_id"),
            "session_id": obj.get("session_id"),
            "profile_id": obj.get("profile_id"),
            "profile_name": obj.get("profile_name"),
            "category": obj.get("category"),
            "action": obj.get("action"),
            "prompt_detected": PromptDetected.from_dict(obj["prompt_detected"])
            if obj.get("prompt_detected") is not None
            else None,
            "response_detected": ResponseDetected.from_dict(obj["response_detected"])
            if obj.get("response_detected") is not None
            else None,
            "prompt_masked_data": MaskedData.from_dict(obj["prompt_masked_data"])
            if obj.get("prompt_masked_data") is not None
            else None,
            "response_masked_data": MaskedData.from_dict(obj["response_masked_data"])
            if obj.get("response_masked_data") is not None
            else None,
            "prompt_detection_details": PromptDetectionDetails.from_dict(obj["prompt_detection_details"])
            if obj.get("prompt_detection_details") is not None
            else None,
            "response_detection_details": ResponseDetectionDetails.from_dict(obj["response_detection_details"])
            if obj.get("response_detection_details") is not None
            else None,
            "tool_detected": ToolDetected.from_dict(obj["tool_detected"])
            if obj.get("tool_detected") is not None
            else None,
            "created_at": obj.get("created_at"),
            "completed_at": obj.get("completed_at"),
        })
        return _obj



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

from singleton_decorator import singleton

from aisecurity.constants.base import (
    MAX_NUMBER_OF_SCAN_IDS,
    MAX_SCAN_ID_STR_LENGTH,
)
from aisecurity.exceptions import AISecSDKException, ErrorType
from aisecurity.generated_openapi_client import ScanIdResult
from aisecurity.generated_openapi_client.urllib3.api.scan_results_api import (
    ScanResultsApi,
)
from aisecurity.generated_openapi_client.urllib3.exceptions import ApiException
from aisecurity.scan.inline.base import ApiBase
from aisecurity.utils import Utils


@singleton
class QueryByScanIds(ApiBase):
    def __init__(self):
        """
        Initialize the QueryByScanIds instance with the ScanResultsApi.
        """
        super().__init__()
        self.__api = ScanResultsApi(api_client=self.api_client)

    def get_scan_results(self, scan_ids: list[str]) -> list[ScanIdResult]:
        """
        Retrieve scan results for the given scan IDs.

        Args:
            scan_ids (list[str]): A list of scan IDs to query.

        Returns:
            list[ScanIdResult]: A list of ScanIdResult objects containing the scan results.

        Raises:
            ValueError: If any scan_id exceeds the maximum allowed length.
        """
        if not scan_ids:
            error_msg = "At least one scan ID must be provided"
            self.logger.error(f"event={self.get_scan_results.__name__} error={error_msg}")
            raise AISecSDKException(
                "At least one scan ID must be provided",
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )

        if len(scan_ids) > MAX_NUMBER_OF_SCAN_IDS:
            error_msg = f"Number of scan IDs exceeds the maximum allowed ({MAX_NUMBER_OF_SCAN_IDS})."
            self.logger.error(f"event={self.get_scan_results.__name__} error={error_msg}")
            raise AISecSDKException(
                error_msg,
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )

        # Validate scan_id lengths
        for scan_id in scan_ids:
            if scan_id is None or len(scan_id) == 0:
                raise AISecSDKException(
                    "Scan Id can't be None or empty",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            elif len(scan_id) > MAX_SCAN_ID_STR_LENGTH:
                error_msg = (
                    f"'{scan_id}' Scan ID exceeds the maximum allowed length of {MAX_SCAN_ID_STR_LENGTH} characters."
                )
                self.logger.error(f"event={self.get_scan_results.__name__} error={error_msg}")
                raise AISecSDKException(
                    error_msg,
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            elif not (Utils.is_valid_uuid(scan_id)):
                error_msg = f"'{scan_id}' Scan ID format must be in UUID format."
                self.logger.error(f"event={self.get_scan_results.__name__} error={error_msg}")
                raise AISecSDKException(error_msg, ErrorType.USER_REQUEST_PAYLOAD_ERROR)

        try:
            response = self.__api.get_scan_results_by_scan_ids(scan_ids=scan_ids)
            return response
        except ApiException as e:
            """
                Handle API exceptions that may occur during the API call. Ex Invalid input data,Authentication errors,Authorization errors,Resource not found
            """
            self.logger.error(f"event={self.get_scan_results.__name__} error={e}")
            raise AISecSDKException(str(e), ErrorType.SERVER_SIDE_ERROR)
        except Exception as e:
            self.logger.error(f"event={self.get_scan_results.__name__} error={e!r}")
            raise AISecSDKException(str(e), ErrorType.AISEC_SDK_ERROR)



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

"""
AISec API service

OpenAPI Specification for the AI Runtime Security API service

The version of the OpenAPI document: 0.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from aisecurity.generated_openapi_client.models.mc_entry_object import McEntryObject
from typing import Optional, Set
from typing_extensions import Self


class McReportObject(BaseModel):
    """
    McReportObject
    """  # noqa: E501

    verdict: Optional[StrictStr] = Field(
        default=None, description='Detection service verdict such as "malicious" or "benign"'
    )
    code_analysis_by_type: Optional[List[McEntryObject]] = None
    all_code_blocks: Optional[List[StrictStr]] = None
    __properties: ClassVar[List[str]] = ["verdict", "code_analysis_by_type", "all_code_blocks"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of McReportObject from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in code_analysis_by_type (list)
        _items = []
        if self.code_analysis_by_type:
            for _item_code_analysis_by_type in self.code_analysis_by_type:
                if _item_code_analysis_by_type:
                    _items.append(_item_code_analysis_by_type.to_dict())
            _dict["code_analysis_by_type"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of McReportObject from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "verdict": obj.get("verdict"),
            "code_analysis_by_type": [McEntryObject.from_dict(_item) for _item in obj["code_analysis_by_type"]]
            if obj.get("code_analysis_by_type") is not None
            else None,
            "all_code_blocks": obj.get("all_code_blocks"),
        })
        return _obj



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

# Header key

import aisecurity._version

HEADER_API_KEY = "x-pan-token"
PAYLOAD_HASH = "x-payload-hash"
USER_AGENT = f"PAN-AIRS/{aisecurity._version.__version__}-python-sdk"

DEFAULT_ENDPOINT = "https://service.api.aisecurity.paloaltonetworks.com"

# Environment variables
AI_SEC_API_KEY = "PANW_AI_SEC_API_KEY"
AI_SEC_API_TOKEN = "PANW_AI_SEC_API_TOKEN"
AI_SEC_API_ENDPOINT = "PANW_AI_SEC_API_ENDPOINT"
HEADER_AUTH_TOKEN = "Authorization"
BEARER = "Bearer "
# Content length limits
MAX_CONTENT_PROMPT_LENGTH = 2 * 1024 * 1024
MAX_CONTENT_RESPONSE_LENGTH = 2 * 1024 * 1024
MAX_CONTENT_CONTEXT_LENGTH = 100 * 1024 * 1024
# Key ID length constraint
MAX_API_KEY_LENGTH = 2048
MAX_TOKEN_LENGTH = 2048


# Request IDs constraints
MAX_TRANSACTION_ID_STR_LENGTH = 100
MAX_SESSION_ID_STR_LENGTH = 100
MAX_SCAN_ID_STR_LENGTH = 36
MAX_NUMBER_OF_SCAN_IDS = 5
MAX_REPORT_ID_STR_LENGTH = 40
MAX_NUMBER_OF_REPORT_IDS = 5
MAX_AI_PROFILE_NAME_LENGTH = 100
MAX_NUMBER_OF_BATCH_SCAN_OBJECTS = 5

# API Client
MAX_CONNECTION_POOL_SIZE = 100
MAX_NUMBER_OF_RETRIES = 5
HTTP_FORCE_RETRY_STATUS_CODES = [500, 502, 503, 504]



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

from urllib3 import poolmanager
from urllib3.util import Retry

from aisecurity import global_configuration
from aisecurity.constants.base import (
    HEADER_API_KEY,
    HTTP_FORCE_RETRY_STATUS_CODES,
    USER_AGENT,
    HEADER_AUTH_TOKEN,
    BEARER,
)
from aisecurity.generated_openapi_client.urllib3.api.scans_api import ScansApi
from aisecurity.generated_openapi_client.urllib3.api_client import ApiClient
from aisecurity.logger import BaseLogger


class ApiBase(BaseLogger):
    def __init__(self):
        super().__init__()
        self.__api_client = self.create_api_client()

    @property
    def api_client(self):
        return self.__api_client

    def create_api_client(self):
        api_client = ApiClient()
        api_client.configuration.logger = self.logger
        api_client.configuration.logger_stream_handler = self.logger.handlers[0]
        api_client.user_agent = USER_AGENT

        api_client.rest_client.pool_manager = poolmanager.PoolManager(
            retries=Retry(
                total=global_configuration.num_retries,
                status_forcelist=HTTP_FORCE_RETRY_STATUS_CODES,
            ),
        )

        if global_configuration.api_endpoint is not None:
            api_client.configuration.host = global_configuration.api_endpoint

        if global_configuration.api_key is not None:
            api_client.set_default_header(HEADER_API_KEY, global_configuration.api_key)
        if global_configuration.api_token is not None:
            api_client.set_default_header(HEADER_AUTH_TOKEN, BEARER + global_configuration.api_token)

        self.logger.debug(f"event={self.create_api_client.__name__} action=new_api_client_created")
        return api_client


class ScanApiBase(ApiBase):
    def __init__(self):
        super().__init__()
        self.__scan_api = ScansApi(self.api_client)

    @property
    def scan_api(self):
        return self.__scan_api
